;DotBlox64d Master
;ATMega8 @ 8 MHz internal R/C

.include "../../include/m168def.inc"

.def	tx_v = R0
.def	tx_h = R1
.def	rx_v = R2
.def	rx_h = R3
.def	clklen = R13
.def	sreg_backup = R14
.def	column = R15

.def	temp = R16
.def	temp2 = R17
.def	count = R18
.def	commtmp = R19
.def	bitcnt = R20
.def	flags = R21
	.equ	fNewData = 0 ;received new data, update display now


.equ	ROW_PORT = PORTD
.equ	COL_PORT = PORTB

;PORTC:
.equ	D_TOP = 4
.equ	D_LEFT = 3
.equ	D_BOTTOM = 2
.equ	D_RIGHT = 1
.equ	CLOCK = 0

;===============================================================================

.dseg
	RAM_Frame: .byte 8 ;one byte per column, MSB = topmost pixel

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
.org OVF2addr
	rjmp	comm
.org OVF0addr
	rjmp	mux
	
;===============================================================================

mux:
	;multiplexing interrupt
	sei ;re-enable interrupts immediately (needed for fast communication)
	push	temp
	push	ZL
	push	ZH
	in	temp, SREG
	push	temp
	
	;select next display column
	inc	column
	sbrc	column, 3
	clr	column

	;read column data from RAM
	ldi	ZL, LOW(RAM_Frame)
	ldi	ZH, HIGH(RAM_Frame)
	add	ZL, column
	brcc	PC+2
	inc	ZH
	ld	temp, Z
	
	;calculate column bit-mask
	ldi	ZL, 0x01
	sbrc	column, 0
	lsl	ZL
	sbrc	column, 1
	lsl	ZL
	sbrc	column, 1
	lsl	ZL
	sbrc	column, 2
	swap	ZL
	com	ZL
	
	;switch to new column
	ldi	ZH, 0xFF
	out	COL_PORT, ZH
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	out	ROW_PORT, temp
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	out	COL_PORT, ZL
	
	pop	temp
	out	SREG, temp
	pop	ZH
	pop	ZL
	pop	temp
	reti
	
;===============================================================================

comm:
	;communications interrupt
	push	temp
	in	temp, SREG
	push	temp
	
	ldi	temp, 256-100
	sts	TCNT2, temp
	
	dec	bitcnt
	breq	comm_done
	
	sbrs	bitcnt, 0
	rjmp	comm_neg_edge
	
	;positive clock edge
	sbi	PORTC, CLOCK
	rjmp	comm_end
	
comm_neg_edge:
	;negative clock edge
	cbi	PORTC, CLOCK
	cbi	PORTC, D_TOP
	sbrc	tx_v, 7
	sbi	PORTC, D_TOP
	lsl	tx_v
	cbi	PORTC, D_LEFT
	sbrc	tx_h, 7
	sbi	PORTC, D_LEFT
	lsl	tx_h
	
comm_end:
	;TODO: read data
	pop	temp
	out	SREG, temp
	pop	temp
	reti
	
comm_done:
	ldi	temp, 0x00
	sts	TIMSK2, temp
	;TODO: flag setzen
	rjmp	comm_end
	
;===============================================================================

reset:
	;set stackpointer
	ldi	temp, HIGH(RAMEND)
	out	SPH, temp
	ldi	temp, LOW(RAMEND)
	out	SPL, temp
	
	;init ports
	ldi	temp, 0xFF
	out	DDRB, temp
	out	PORTB, temp
	ldi	temp, 1<<D_TOP | 1<<D_LEFT | 1<<CLOCK
	out	DDRC, temp
	ldi	temp, 0x21
	out	PORTC, temp
	ldi	temp, 0xFF
	out	DDRD, temp
	ldi	temp, 0x00
	out	PORTD, temp
	
	;init multiplexing timer
	ldi	temp, 0x02 ;clk/8
	out	TCCR0B, temp
	ldi	temp, 1<<TOIE0
	sts	TIMSK0, temp
	
	;init clock pulse timer
	ldi	temp, 0x01 ;Clk/1
	sts	TCCR2B, temp
	
	;init RAM
	ldi	ZL, LOW(RAM_Frame)
	ldi	ZH, HIGH(RAM_Frame)
	ldi	temp, 0x00
	ldi	temp2, 8
init_ram:
	st	Z+, temp
	dec	temp2
	brne	init_ram
	
	;init registers
	clr	column
	clr	bitcnt
	
	;send reset
	ldi	temp, 0
	ldi	temp2, 0
init_wait1:
	dec	temp2
	brne	init_wait1
	dec	temp
	brne	init_wait1
	cbi	PORTC, CLOCK ;long clock pulse (reset)
	ldi	temp, 0
	ldi	temp2, 0
init_wait2:
	dec	temp2
	brne	init_wait2
	dec	temp
	brne	init_wait2
	sbi	PORTC, CLOCK
	ldi	temp, 0
	sts	TCNT2, temp
	ldi	temp, 1<<TOV2
	out	TIFR2, temp
	ldi	temp, 1<<TOIE2
	sts	TIMSK2, temp
	ldi	bitcnt, 2+1
	
	;enable interrupts
	sei
	
;-------------------------------------------------------------------------------

loop:
	ldi	temp, 0xFF
	rcall	sendbyte
	ldi	temp, 0x10
	rcall	sendbyte
	ldi	temp, 0x10
	rcall	sendbyte
	ldi	temp, 0x10
	rcall	sendbyte
	ldi	temp, 0xFF
	rcall	sendbyte
	ldi	temp, 0x00
	rcall	sendbyte
	ldi	temp, 0x7F
	rcall	sendbyte
	ldi	temp, 0x90
	rcall	sendbyte
	ldi	temp, 0x90
	rcall	sendbyte
	ldi	temp, 0x90
	rcall	sendbyte
	ldi	temp, 0x7F
	rcall	sendbyte
	ldi	temp, 0x00
	rcall	sendbyte
	ldi	temp, 0xFF
	rcall	sendbyte
	ldi	temp, 0x90
	rcall	sendbyte
	ldi	temp, 0x90
	rcall	sendbyte
	ldi	temp, 0x90
	rcall	sendbyte
	ldi	temp, 0x6F
	rcall	sendbyte
	ldi	temp, 0x00
	rcall	sendbyte
	ldi	temp, 0x00
	rcall	sendbyte
	ldi	temp, 0x00
	rcall	sendbyte
	ldi	temp, 0x00
	rcall	sendbyte
	rjmp	loop
	
;===============================================================================

sendbyte:
	;send bytes in tx_v and tx_h
	cpi	bitcnt, 0 ;wait until line is clear
	brne	sendbyte
	
	push	temp
	ldi	ZH, HIGH(RAM_Frame)
	ldi	ZL, LOW(RAM_Frame)
	ld	tx_h, Z
	ldi	temp2, 7
shift:
	ldd	temp, Z+1
	st	Z+, temp
	dec	temp2
	brne	shift
	pop	temp
	st	Z, temp
	
	ldi	temp, 256-200 ;send SYNC pulse, followed by 8 data bits
	sts	TCNT2, temp
	cbi	PORTC, CLOCK
	ldi	temp, 1<<TOV2
	out	TIFR2, temp
	ldi	temp, 1<<TOIE2
	sts	TIMSK2, temp
	ldi	bitcnt, 17+1
	
	ldi	count, 5
	ldi	temp, 0
	ldi	temp2, 0
wait:
	dec	temp2
	brne	wait
	dec	temp
	brne	wait
	dec	count
	brne	wait
	
	ret
	