;DotBlox64d Slave
;ATMega8 @ 8 MHz internal R/C

.include "../../include/m168def.inc"

.def	tx_v = R0
.def	tx_h = R1
.def	rx_v = R2
.def	rx_h = R3
.def	clklen = R13
.def	sreg_backup = R14
.def	column = R15

.def	temp = R16
.def	temp2 = R17
.def	count = R18
.def	commtmp = R19
.def	bitcnt = R20
.def	flags = R21
	.equ	fNewData = 0 ;received new data, update display now


.equ	ROW_PORT = PORTD
.equ	COL_PORT = PORTB

;PORTC:
.equ	D_TOP = 4
.equ	D_LEFT = 3
.equ	D_BOTTOM = 2
.equ	D_RIGHT = 1
.equ	CLOCK = 0

;===============================================================================

.dseg
	RAM_Frame: .byte 8 ;one byte per column, MSB = topmost pixel

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
.org PCI1addr
	rjmp	pcint
.org OVF0addr
	rjmp	mux
	
;===============================================================================

pcint:
	;clock line pin change (do not use temp/... here, only commtmp)
	in	sreg_backup, SREG
	
	sbis	PINC, 0
	rjmp	clk_falling_edge
	
	;rising edge => receive data
	lds	commtmp, TCNT2
	sbic	TIFR2, TOV2
	rjmp	clk_reset
	cpi	bitcnt, 0xFF
	breq	clk_calclen
	cp	commtmp, clklen
	brsh	clk_sync
	cpi	bitcnt, 0 ;shouldn't happen
	breq	clk_end
	lsl	rx_v ;shift in vertical data
	sbic	PINC, D_BOTTOM
	inc	rx_v
	lsl	rx_h ;shift in horizontal data
	sbic	PINC, D_RIGHT
	inc	rx_h
	dec	bitcnt
	brne	PC+2
	ori	flags, 1<<fNewData
	;READ DATA -- TODO
	rjmp	clk_end
	
clk_sync:
	;received sync pulse
	ldi	bitcnt, 8
	rjmp	clk_end
	
clk_falling_edge:
	;falling edge => write data, clear clock pulse length timer
	ldi	commtmp, 0
	sts	TCNT2, commtmp
	ldi	commtmp, 1<<TOV2
	out	TIFR2, commtmp
	cpi	bitcnt, 0
	breq	clk_end
	cbi	PORTC, D_TOP ;shift out vertical data
	sbrc	tx_v, 7
	sbi	PORTC, D_TOP
	lsl	tx_v
	cbi	PORTC, D_LEFT ;shift out horizontal data
	sbrc	tx_h, 7
	sbi	PORTC, D_LEFT
	lsl	tx_h
	
clk_end:
	;return
	out	SREG, sreg_backup
	reti
	
;--------------------
	
clk_calclen:
	mov	clklen, commtmp
	lsr	commtmp
	add	clklen, commtmp
	clr	bitcnt
	rjmp	clk_end

clk_reset:
	ldi	bitcnt, 0xFF
	rjmp	clk_end

;===============================================================================

mux:
	;multiplexing interrupt
	sei ;re-enable interrupts immediately (needed for fast communication)
	push	temp
	push	ZL
	push	ZH
	in	temp, SREG
	push	temp
	
	;select next display column
	inc	column
	sbrc	column, 3
	clr	column

	;read column data from RAM
	ldi	ZL, LOW(RAM_Frame)
	ldi	ZH, HIGH(RAM_Frame)
	add	ZL, column
	brcc	PC+2
	inc	ZH
	ld	temp, Z
	
	;calculate column bit-mask
	ldi	ZL, 0x01
	sbrc	column, 0
	lsl	ZL
	sbrc	column, 1
	lsl	ZL
	sbrc	column, 1
	lsl	ZL
	sbrc	column, 2
	swap	ZL
	com	ZL
	
	;switch to new column
	ldi	ZH, 0xFF
	out	COL_PORT, ZH
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	out	ROW_PORT, temp
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	out	COL_PORT, ZL
	
	pop	temp
	out	SREG, temp
	pop	ZH
	pop	ZL
	pop	temp
	reti
	
;===============================================================================

reset:
	;set stackpointer
	ldi	temp, HIGH(RAMEND)
	out	SPH, temp
	ldi	temp, LOW(RAMEND)
	out	SPL, temp
	
	;init ports
	ldi	temp, 0xFF
	out	DDRB, temp
	out	PORTB, temp
	ldi	temp, 1<<D_TOP | 1<<D_LEFT
	out	DDRC, temp
	ldi	temp, 0x21
	out	PORTC, temp
	ldi	temp, 0xFF
	out	DDRD, temp
	ldi	temp, 0x00
	out	PORTD, temp
	
	;init multiplexing timer
	ldi	temp, 0x02 ;clk/8
	out	TCCR0B, temp
	ldi	temp, 1<<TOIE0
	sts	TIMSK0, temp
	
	;init clock pulse length timer
	ldi	temp, 0x01 ;Clk/1
	sts	TCCR2B, temp	
	
	;init pin change interrupt for clock line (PC0)
	ldi	temp, 1<<PCIE1
	sts	PCICR, temp
	ldi	temp, 0x01
	sts	PCMSK1, temp
	
	;init RAM
	ldi	ZL, LOW(RAM_Frame)
	ldi	ZH, HIGH(RAM_Frame)
	ldi	temp, 0x00
	ldi	temp2, 8
init_ram:
	st	Z+, temp
	dec	temp2
	brne	init_ram
	
	;init registers
	clr	column
	clr	bitcnt
	clr	tx_h
	clr	tx_v
	
	;enable interrupts
	sei
	
;-------------------------------------------------------------------------------

loop:
	;wait for new data
	sbrs	flags, fNewData
	rjmp	loop
	
	ldi	temp2, 7
	ldi	ZH, HIGH(RAM_Frame)
	ldi	ZL, LOW(RAM_Frame)
	ldd	tx_h, Z+1
shift:
	ldd	temp, Z+1
	st	Z+, temp
	dec	temp2
	brne	shift
	st	Z, rx_h
	andi	flags, ~(1<<fNewData)

	rjmp	loop
	